package de.kaffeemitkoffein.imagepipe;

import android.app.Activity;
import android.content.Context;
import android.os.Bundle;
import android.os.PersistableBundle;
import android.view.View;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.LinearLayout;

public class ExifMenu extends Activity {

    private Context context;
    private ExifData exifData;
    private String[] allowedTags;

    @Override
    protected void onResume() {
        super.onResume();
    }

    @Override
    protected void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);
    }

    @Override
    public void onSaveInstanceState(Bundle outState, PersistableBundle outPersistentState) {
        super.onSaveInstanceState(outState, outPersistentState);
    }

    @Override
    protected void onPause() {
        super.onPause();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
    }

    private boolean isAllowedTag(String tag){
        for (int i=0; i<allowedTags.length; i++){
            if (allowedTags[i].equalsIgnoreCase(tag)){
                return true;
            }
        }
        return false;
    }

    private int getTagPosition(String tag){
        for (int i=0; i<exifData.exifItems.size(); i++){
            if (exifData.exifItems.get(i).tag.equalsIgnoreCase(tag)){
                return i;
            }
        }
        return -1;
    }

    private void setTagValue(String tag, boolean state){
        int pos = getTagPosition(tag);
        if (pos>=0){
            exifData.exifItems.get(pos).isAllowed = state;
        }
    }

    private String generateAllowedString(){
        StringBuilder stringBuilder = new StringBuilder();
        for (int i=0; i<exifData.exifItems.size(); i++){
            ExifData.ExifItem exifItem = exifData.exifItems.get(i);
            if (exifItem.isAllowed){
                stringBuilder.append(exifItem.tag);
                stringBuilder.append(ImagepipePreferences.TAG_SEPERATOR);
            }
        }
        // remove last # if string not empty
        if (stringBuilder.length()>0){
            stringBuilder.setLength(stringBuilder.length()-1);
        }
        return stringBuilder.toString();
    }

    private void addCheckBox(LinearLayout parent, ExifData.ExifItem exifItem){
        CheckBox checkBox = new CheckBox(this);
        checkBox.setText(exifItem.description);
        checkBox.setChecked(isAllowedTag(exifItem.tag));
        checkBox.setTag(exifItem);
        checkBox.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton compoundButton, boolean state) {
                ExifData.ExifItem exifItem = (ExifData.ExifItem) compoundButton.getTag();
                setTagValue(exifItem.tag,state);
                ImagepipePreferences.setPrefAllowedTags(context,generateAllowedString());
                ImagepipePreferences.setAllowedTagsModified(context);
            }
        });
        parent.addView(checkBox);
    }

    @Override
    protected void onCreate(Bundle bundle) {
        super.onCreate(bundle);
        context = getApplicationContext();
        setContentView(R.layout.activity_exifmenu);
        exifData = new ExifData(context,null);
        allowedTags = ImagepipePreferences.getAllowedTags(context);
        LinearLayout linearLayout = (LinearLayout) findViewById(R.id.exifmenu_linearlayout);
        for (int i=0; i<exifData.exifItems.size(); i++){
            addCheckBox(linearLayout,exifData.exifItems.get(i));
        }
        Button resetButton = (Button) findViewById(R.id.exifmenu_resetbutton);
        resetButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                ImagepipePreferences.setPrefAllowedTags(context,"");
                recreate();
            }
        });
    }
}
